@@ -100,10 +100,10 @@ int SDPCmdBase::init_cmd()
 	return 0;
 }
 
-IvtHeader *SDPCmdBase::search_ivt_header(shared_ptr<FileBuffer> data, size_t &off, size_t limit)
+IvtHeader *SDPCmdBase::search_ivt_header(shared_ptr<DataBuffer> data, size_t &off, size_t limit)
 {
-	if (limit >= data->m_avaible_size)
-		limit = data->m_avaible_size;
+	if (limit >= data->size())
+		limit = data->size();
 
 	for (; off < limit; off += 0x4)
 	{
@@ -149,12 +149,14 @@ int SDPDcdCmd::run(CmdCtx*ctx)
 	}
 	init_cmd();
 
-	shared_ptr<FileBuffer> buff, p = get_file_buffer(m_filename, true);
+	shared_ptr<FileBuffer> p = get_file_buffer(m_filename, true);
 
 	if (!p)
 		return -1;
 
+	shared_ptr<DataBuffer> buff;
 	buff = p->request_data(0, m_scan_limited);
+	if (!buff) return -1;
 
 	size_t off = 0;
 	IvtHeader *pIVT = search_ivt_header(buff, off);
@@ -184,7 +186,7 @@ int SDPDcdCmd::run(CmdCtx*ctx)
 		return -1;
 	}
 
-	// point maybe change after new requestion buffer.
+	// point maybe change after new requisition buffer.
 	pdcd = &(buff->at(off + pIVT->DCDAddress - pIVT->SelfAddr));
 
 	m_spdcmd.m_cmd = ROM_KERNEL_CMD_DCD_WRITE;
@@ -357,14 +359,17 @@ int SDPWriteCmd::run(CmdCtx*ctx)
 {
 	size_t size;
 	uint8_t *pbuff;
-	int offset = 0;
+	ssize_t offset = 0;
 
-	shared_ptr<FileBuffer> fbuff, p1= get_file_buffer(m_filename, true);
+	shared_ptr<FileBuffer> p1= get_file_buffer(m_filename, true);
 
 	if (p1 == nullptr)
 		return -1;
 
+	shared_ptr<DataBuffer> fbuff;
+
 	fbuff = p1->request_data(0, m_scan_limited);
+	if (!fbuff) return -1;
 
 	if (m_Ivt < 0)
 	{
@@ -410,15 +415,18 @@ int SDPWriteCmd::run(CmdCtx*ctx)
 			IvtHeader *pIvt = search_ivt_header(fbuff, off, 0x100000);
 			if (pIvt)
 			{
-				BootData *pDB = (BootData *) &(fbuff->at(off + pIvt->BootData - pIvt->SelfAddr));
-				offset = off + pDB->ImageSize - (pIvt->SelfAddr - pDB->ImageStartAddr);
+				if (pIvt->BootData)
+				{
+					BootData *pDB = (BootData *) &(fbuff->at(off + pIvt->BootData - pIvt->SelfAddr));
+					offset = off + pDB->ImageSize - (pIvt->SelfAddr - pDB->ImageStartAddr);
+				}
 			}
 			else
 			{
 				offset += GetContainerActualSize(fbuff, offset);
 			}
 
-			if (offset >= fbuff->m_avaible_size)
+			if (size_t(offset) >= fbuff->size())
 			{
 				set_last_err_string("Unknown Image type, can't use skipspl format");
 				return -1;
@@ -672,11 +680,13 @@ int SDPJumpCmd::run(CmdCtx *ctx)
 		return 0;
 	}
 
-	shared_ptr<FileBuffer> buff, p1 = get_file_buffer(m_filename, true);
+	shared_ptr<FileBuffer> p1 = get_file_buffer(m_filename, true);
 	if (!p1)
 		return -1;
 
+	shared_ptr<DataBuffer> buff;
 	buff = p1->request_data(0, m_scan_limited);
+	if (!buff) return -1;
 
 	size_t off = 0;
 	IvtHeader *pIVT = search_ivt_header(buff, off, m_scan_limited);
